@@ -2,138 +2,119 @@
 
 # Title:  Chain-linking, rebasing and index frequency conversion
 # Author: Sebastian Weinand
-# Date:   7 February 2024
+# Date:   2 June 2024
 
 # unchain monthly time series:
 unchain <- function(x, t, by=12){
-
+  
   # input checks:
   check.num(x=x)
   check.date(x=t, na.ok=FALSE)
   check.lengths(x=x, y=t)
   check.num(x=by, min.len=1, max.len=1, null.ok=TRUE, int=c(1,12))
-
+  
   # extract year and month:
   t <- as.Date(x=t, format="%Y-%m-%d")
   y <- as.integer(format(t, "%Y"))
   m <- as.integer(format(t, "%m"))
-
+  
   if(is.null(by)){
-
-    # compute shifted average:
-    x.ref.shifted <- tapply(X=x, INDEX=y+1, FUN=mean, na.rm=TRUE)
-
-    # use only full calendar years:
-    x.ref.shifted[tapply(X=x, INDEX=y+1, FUN=length)<12] <- NA
-
-    # match index values of reference period to other index values:
-    x.ref.shifted <- x.ref.shifted[match(x=y, table=names(x.ref.shifted))]
-
+    
+    # compute shifted annual average using full calendar years:
+    x.ref <- tapply(X=x, INDEX=y+1, FUN=function(z) if(all(is.na(z)) | length(z)<12L){NA_real_}else{mean(z, na.rm=TRUE)})
+    
+    # match index values of reference period to time periods:
+    x.ref.shifted <- x.ref[match(x=y, table=names(x.ref))]
+    
   }else{
-
+    
+    # index value in price reference period:
+    x.ref <- x[m==by]
+    
     # derive reference time period:
-    t.ref <- paste(y-1+ifelse(m>by,1,0), by, sep="-")
-
-    # match index values of reference period to other index values:
-    x.ref.shifted <- (x[m==by])[match(x=t.ref, table=paste(y[m==by], by, sep="-"))]
-
+    y.ref <- paste(y-1+ifelse(m>by,1,0), by, sep="-")
+    
+    # match index values of reference period to time periods:
+    x.ref.shifted <- x.ref[match(x=y.ref, table=paste(y[m==by], by, sep="-"))]
+    
   }
-
+  
   # unchain index:
   x.unchained <- as.vector(x/x.ref.shifted)
-
+  
   # return output:
   return(x.unchained)
-
+  
 }
 
 # chain monthly time series:
 chain <- function(x, t, by=12){
-
+  
   # input checks:
   check.num(x=x)
   check.date(x=t, na.ok=FALSE)
   check.lengths(x=x, y=t)
   check.num(x=by, min.len=1, max.len=1, null.ok=TRUE, int=c(1,12))
-
+  
   # extract year and month:
   t <- as.Date(x=t, format="%Y-%m-%d")
   y <- as.integer(format(t, "%Y"))
   m <- as.integer(format(t, "%m"))
-
+  
   if(is.null(by)){
-
-    # subset to relevant months:
-    x.ref <- tapply(X=x, INDEX=y, FUN=mean, na.rm=TRUE)
-
-    # make sure that only full calendar years are considered:
-    x.ref[tapply(X=x, INDEX=y, FUN=length)<12] <- NA
-
-    # years that contain only NAs indicate break in time series.
-    # cumulative growth rates should start from 1 again after
-    # each break. thus, time groups are derived:
-    y.ref.grp <- cumsum(tapply(X=x, INDEX=y+1, FUN=function(z) all(is.na(z))))
-
-    # cumulative growth rates per time group:
-    x.ref.cum <- unlist(
-      x=tapply(x.ref, y.ref.grp, FUN=function(z) cumprod(ifelse(is.na(z), 1, z))),
-      use.names=FALSE)
-
-    # set names:
-    names(x.ref.cum) <- names(y.ref.grp)
-
-    # replicate cumulative growth rates:
-    x.ref.cum.shifted <- x.ref.cum[match(x=y, table=names(x.ref.cum))]
-
-    # chain_annual series:
-    x.chained <- 100*as.vector(x*x.ref.cum.shifted)
-
-    # set index start value to 100:
-    y.base <- which(diff(tapply(X=x.chained, INDEX=y-1, FUN=function(z) all(is.na(z))))<0)
-    x.chained[y%in%names(y.base)] <- 100
-
+    
+    # align years to previous price reference periods:
+    y.ref <- y-1
+    
+    # compute annual average using full calendar years:
+    x.ref <- tapply(X=x, INDEX=y, FUN=function(z) if(all(is.na(z)) | length(z)<12L){NA_real_}else{mean(z, na.rm=TRUE)})
+    
+    # match annual averages to years:
+    x.ref <- x.ref[match(x=unique(y.ref), table=names(x.ref))]
+    
+    # overwrite for checking in last step:
+    by <- m
+    
   }else{
-
-    # frequency of price reference years:
+    
+    # align years to price reference month:
     y.ref <- y-1+ifelse(m>by, 1, 0)
-    y.ref.tab <- table(y.ref)
-
-    # subset to relevant months:
-    x.ref <- x[m==by]
-
-    # set names:
-    names(x.ref) <- format(t[m==by], "%Y")
-
-    # match with years:
-    x.ref <- x.ref[match(names(y.ref.tab), names(x.ref))]
-
-    # set names:
-    names(x.ref) <- names(y.ref.tab)
-
-    # years that contain only NAs indicate break in time series.
-    # cumulative growth rates should start from 1 again after
-    # each break. thus, time groups are derived:
-    y.ref.grp <- cumsum(tapply(x, y.ref, FUN=function(z) all(is.na(z))))
-
-    # cumulative growth rates per time group:
-    x.ref.cum <- unlist(
-      x=tapply(x.ref, y.ref.grp, FUN=function(z) cumprod(ifelse(is.na(z), 1, z))),
-      use.names=FALSE)
-
-    # replicate cumulative growth rates:
-    x.ref.cum.shifted <- rep(x=x.ref.cum, y.ref.tab)
-
-    # chain series:
-    x.chained <- 100*as.vector(x*x.ref.cum.shifted)
-
-    # set index start value to 100:
-    x.chained[c(diff(is.na(x.chained)),0)<0] <- 100
-
+    
+    # match price reference indices to years:
+    x.ref <- x[match(x=paste(unique(y.ref), by, sep="-"), table=paste(y,m,sep="-"))]
+    
   }
-
+  
+  # overwrite names:
+  names(x.ref) <- unique(y.ref)
+  
+  # years that contain only NAs indicate break in time series.
+  # cumulative growth rates should start from 1 again after
+  # each break. thus, time groups are derived:
+  y.ref.grp <- cumsum(tapply(X=x, INDEX=y.ref, FUN=function(z) all(is.na(z))))
+  
+  # cumulative growth rates per time group:
+  x.ref.cum <- tapply(x.ref, y.ref.grp, FUN=function(z) cumprod(ifelse(is.na(z), 1, z)))
+  
+  # derive base years where index value is 100:
+  y.base <- sapply(X=x.ref.cum[lengths(x.ref.cum)>1], FUN=function(z) names(z)[2])
+  
+  # unlist and set names:
+  x.ref.cum <- unlist(x=x.ref.cum, use.names=FALSE)
+  names(x.ref.cum) <- names(y.ref.grp)
+  
+  # match cumulative growth rates to months:
+  x.ref.cum.shifted <- x.ref.cum[match(x=y.ref, table=names(x.ref.cum))]
+  
+  # chain index series:
+  x.chained <- 100*as.vector(x*x.ref.cum.shifted)
+  
+  # set index start value to 100:
+  x.chained[y%in%y.base & m==by] <- 100
+  
   # print unchained values:
   return(x.chained)
-
+  
 }
 
 # rebase index series:
